@@ -1,38 +1,28 @@
 #include <windows.h>
-#include "../ExDLL/exdll.h"
+#include <nsis/pluginapi.h> // nsis plugin
 #include "../../Source/exehead/resource.h"
 
-// Turn a pair of chars into a word
-// Turn four chars into a dword
-#ifdef __BIG_ENDIAN__ // Not very likely, but, still...
-#define CHAR2_TO_WORD(a,b) (((WORD)(b))|((a)<<8))
-#define CHAR4_TO_DWORD(a,b,c,d)	(((DWORD)CHAR2_TO_WORD(c,d))|(CHAR2_TO_WORD(a,b)<<16))
-#else
-#define CHAR2_TO_WORD(a,b) (((WORD)(a))|((b)<<8))
-#define CHAR4_TO_DWORD(a,b,c,d)	(((DWORD)CHAR2_TO_WORD(a,b))|(CHAR2_TO_WORD(c,d)<<16))
-#endif
+// Unicode support added by Jim Park -- 08/02/2007
 
 HINSTANCE hInstance;
 HWND hwBanner;
 HANDLE hThread;
 BOOL bFailed;
 
-char buf[1024];
+TCHAR buf[1024];
 
-unsigned int myatoi(char *s);
-
-BOOL CALLBACK BannerProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
+INT_PTR CALLBACK BannerProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
   if (uMsg == WM_INITDIALOG)
   {
     int iMainStringSet = 0;
 
     popstring(buf);
-    while (*(int*)buf == CHAR4_TO_DWORD('/','s','e','t') && !buf[4])
+    while (lstrcmp(buf, _T("/set")) == 0)
     {
       unsigned int id;
       popstring(buf);
-      id = myatoi(buf);
+      id = myatou(buf);
       popstring(buf);
       SetDlgItemText(hwndDlg, id, buf);
       popstring(buf);
@@ -46,13 +36,13 @@ BOOL CALLBACK BannerProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
       SetDlgItemText(hwndDlg, IDC_STR, buf);
 
     if (!*buf)
-      SetWindowLong(hwndDlg, GWL_EXSTYLE, GetWindowLong(hwndDlg, GWL_EXSTYLE) | WS_EX_TOOLWINDOW);
+      SetWindowLongPtr(hwndDlg, GWL_EXSTYLE, GetWindowLongPtr(hwndDlg, GWL_EXSTYLE) | WS_EX_TOOLWINDOW);
   }
   if (uMsg == WM_CLOSE)
   {
     DestroyWindow(hwndDlg);
   }
-  return 0;
+  return FALSE;
 }
 
 BOOL ProcessMessages()
@@ -74,6 +64,19 @@ DWORD WINAPI BannerThread(LPVOID lpParameter)
   HWND hwndParent = (HWND) lpParameter;
   HWND lhwBanner;
 
+  // This right here is the mother of all evils when it comes to
+  // foreground windows. The dialog is created in another thread
+  // and there can only be one thread holding the right to set the
+  // foreground window. So long as this thread exists and has an
+  // active window, another thread from the same process can steal
+  // its thunder. But if the window and the thread are destroyed,
+  // the foreground rights pass on to another process. To avoid
+  // this situation that could cause the installer to show up on
+  // the background if Banner is used in .onInit, we don't let
+  // CreateDialog show the window and instead do this in the
+  // original thread. This is done by not specifying WS_VISIBLE
+  // for IDD_VERIFY.
+
   lhwBanner = CreateDialog(
     GetModuleHandle(0),
     MAKEINTRESOURCE(IDD_VERIFY),
@@ -100,10 +103,17 @@ DWORD WINAPI BannerThread(LPVOID lpParameter)
   return 0;
 }
 
-void __declspec(dllexport) show(HWND hwndParent, int string_size, char *variables, stack_t **stacktop)
+static UINT_PTR PluginCallback(enum NSPIM msg)
+{
+  return 0;
+}
+
+void __declspec(dllexport) show(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop, extra_parameters *extra)
 {
   EXDLL_INIT();
 
+  extra->RegisterPluginCallback(hInstance, PluginCallback);
+
   {
     DWORD dwThreadId;
     DWORD dwMainThreadId = GetCurrentThreadId();
@@ -117,7 +127,7 @@ void __declspec(dllexport) show(HWND hwndParent, int string_size, char *variable
 
     hThread = CreateThread(0, 0, BannerThread, (LPVOID) hwndParent, 0, &dwThreadId);
 
-    // wait for the window to initalize and for the stack operations to finish
+    // wait for the window to initialize and for the stack operations to finish
     while (hThread && !hwBanner && !bFailed)
     {
       ProcessMessages();
@@ -137,13 +147,13 @@ void __declspec(dllexport) show(HWND hwndParent, int string_size, char *variable
   }
 }
 
-void __declspec(dllexport) getWindow(HWND hwndParent, int string_size, char *variables, stack_t **stacktop)
+void __declspec(dllexport) getWindow(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop)
 {
-  wsprintf(buf, "%u", hwBanner);
+  wsprintf(buf, _T("%u"), hwBanner);
   pushstring(buf);
 }
 
-void __declspec(dllexport) destroy(HWND hwndParent, int string_size, char *variables, stack_t **stacktop)
+void __declspec(dllexport) destroy(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop)
 {
   if (!hwBanner)
     return;
@@ -158,7 +168,7 @@ void __declspec(dllexport) destroy(HWND hwndParent, int string_size, char *varia
   }
 }
 
-BOOL WINAPI DllMain(HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
+BOOL WINAPI DllMain(HINSTANCE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
 {
   hInstance = hInst;
   if (hwBanner && ul_reason_for_call == DLL_PROCESS_DETACH)
@@ -167,18 +177,3 @@ BOOL WINAPI DllMain(HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
   }
   return TRUE;
 }
-
-unsigned int myatoi(char *s)
-{
-  unsigned int v=0;
-
-  for (;;)
-  {
-    unsigned int c=*s++;
-    if (c >= '0' && c <= '9') c-='0';
-    else break;
-    v*=10;
-    v+=c;
-  }
-  return v;
-}
