@@ -3,7 +3,7 @@
 ** Copyright (C) 2000-2001 Nullsoft, Inc.
 ** Author: Justin Frankel
 ** File: util.h - JNL interface for basic network utilities
-** License: see jnetlib.h
+** License: see License.txt
 **
 ** routines you may be interested in:
 **   JNL::open_socketlib(); 
@@ -19,16 +19,18 @@
 **    gives you the integer representation of a ip address in dotted 
 **    decimal form.
 **
-**  JNL::addr_to_ipstr(unsigned long addr, char *host, int maxhostlen);
+**   JNL::addr_to_ipstr(unsigned long addr, char *host, int maxhostlen);
 **    gives you the dotted decimal notation of an integer ip address.
 **
+** Reviewed for Unicode support by Jim Park -- 08/24/2007
+**   Keep the functions here strictly ANSI.
 */
 
 #ifndef _UTIL_H_
 #define _UTIL_H_
 
 int my_atoi(char *p);
-__int64 myatoi64(char *s);
+__int64 myatoi64(const char *s);
 void myitoa64(__int64 i, char *buffer);
 void mini_memset(void *,char,int);
 void mini_memcpy(void *,void*,int);
