@@ -1,8 +1,9 @@
 #include <windows.h>
 #include <commctrl.h>
 
+#include <nsis/pluginapi.h> // nsis plugin
+
 #include "defs.h"
-#include "nsis.h"
 
 #ifndef WS_EX_RIGHT
 #  define WS_EX_RIGHT 0x1000
@@ -22,7 +23,7 @@
 
 extern struct nsDialog g_dialog;
 
-void __declspec(dllexport) SetRTL(HWND hwndParent, int string_size, char *variables, stack_t **stacktop, extra_parameters *extra)
+void __declspec(dllexport) SetRTL(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop, extra_parameters *extra)
 {
   g_dialog.rtl = (BOOL) popint();
 }
@@ -34,6 +35,7 @@ void NSDFUNC ConvertStyleToRTL(enum nsControlType type, LPDWORD style, LPDWORD e
 
   switch (type)
   {
+  case NSCTL_LINK:
   case NSCTL_BUTTON:
     *style ^= BS_LEFTTEXT | BS_RIGHT | BS_LEFT;
 
