@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2002 Robert Rainwater <rrainwater@yahoo.com>
-Copyright (C) 2002-2019 Nullsoft and Contributors
+Copyright (C) 2002-2025 Nullsoft and Contributors
 
 This software is provided 'as-is', without any express or implied
 warranty.  In no event will the authors be held liable for any damages
@@ -181,11 +181,11 @@ void ExecScript(int mode) {
   int ignoreData = mode == MODE_IGNOREOUTPUT;
   int logMode = mode == MODE_LINES, stackMode = mode == MODE_STACK;
   unsigned int to, tabExpandLength = logMode ? TAB_REPLACE_CCH : 0, codepage;
-  BOOL bOEM;
+  BOOL bOEM, forceNarrowInput = FALSE;
 
   *szRet = _T('\0');
 
-  if (!IsWOW64()) {
+  if (sizeof(void*) < 8 && !IsWOW64()) {
     TCHAR* p;
     int nComSpecSize;
 
@@ -227,7 +227,11 @@ void ExecScript(int mode) {
         pNTHeaders = (PIMAGE_NT_HEADERS)(pMapView + ((PIMAGE_DOS_HEADER)pMapView)->e_lfanew);
         // Turning the copied DLL into a stripped down executable.
         pNTHeaders->FileHeader.Characteristics = IMAGE_FILE_32BIT_MACHINE | IMAGE_FILE_LOCAL_SYMS_STRIPPED | 
-          IMAGE_FILE_LINE_NUMS_STRIPPED | IMAGE_FILE_EXECUTABLE_IMAGE;
+          IMAGE_FILE_LINE_NUMS_STRIPPED | IMAGE_FILE_EXECUTABLE_IMAGE
+        #ifdef _WIN64
+        | IMAGE_FILE_LARGE_ADDRESS_AWARE // For ARM64 (feature request #579)
+        #endif
+        ;
         // Windows character-mode user interface (CUI) subsystem.
         pNTHeaders->OptionalHeader.Subsystem = IMAGE_SUBSYSTEM_WINDOWS_CUI;
         // g_hInst is assumed to be the very base of the DLL in memory.
@@ -275,7 +279,12 @@ params:
     goto params;
   }
   if (!lstrcmpi(pExec, _T("/OEM"))) {
-    bOEM = TRUE;
+    bOEM = forceNarrowInput = TRUE;
+    *pExec = 0;
+    goto params;
+  }
+  if (!lstrcmpi(pExec, _T("/MBCS"))) {
+    forceNarrowInput = TRUE;
     *pExec = 0;
     goto params;
   }
@@ -300,7 +309,7 @@ params:
     const BOOL isNT = sizeof(void*) > 4 || (GetVersion() < 0x80000000);
     HANDLE newstdout = 0, read_stdout = 0;
     HANDLE newstdin = 0, read_stdin = 0;
-    int utfSource = sizeof(TCHAR) > 1 ? -1 : FALSE, utfOutput = sizeof(TCHAR) > 1;
+    int utfSource = sizeof(TCHAR) > 1 && !forceNarrowInput ? -1 : FALSE, utfOutput = sizeof(TCHAR) > 1;
     DWORD cbRead, dwLastOutput;
     DWORD dwExit = 0, waitResult = WAIT_TIMEOUT;
     static BYTE bufSrc[1024];
